--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: Antti Lukats, Oleksandr Kiyenko, John Hartfiel
--
-- TEBF808 Slave CPLD U39, SC.Sch, second in the JTAG Chain
--
--
-- Revision     : REV07
-- Release Date : 2019-05-03
-- Supported PCB Revision: 03,04
-- changes REV06 to REV07:
--  - complete rework
--  - add variants (power up and SD)
--  - multi-functions for buttons
--  - power on and power downs sequencing 
--        - module complete disable on power down
--        - power on sequencing
--        - power down sequencing --> can be forced with power button (hold longer)
--  - Soft PS or PS POR Reset on Reset button (hold longer for PS POR Reset)
--  - add inter CPLD RGPIO
--  - new SoC RGPIO Pinout
--  - removed reboot for pcie initialization
--  - new LED debugging sequencing
--  - Disabled UART on power down state
--  - bugfix WP pin for microSD slot
--  - removed PCIe Reboot.
-- changes REV05 to REV06:
--  - new LED D2 D3 only power + Reset
--  - HD_LED status PS ERRORS
--  - extended Power Management
-- changes REV04 to REV05:
--  - PS can be rebooted by FSBL over MIO30, for proper pcie initialization without Reset Button
--  - SD Boot from micoSD only if switch is selected  
--  - RGPIO connection 
--  - SD WP to FPGA
--  - debounce power/rst button
--  - MIO and PL LED access
--  - Needed changes since Lattice Diamond 3.9 (change date 2017.07.10) (CPLD Revision is not changed for this source code bugfix)
--    - rename label for Reset/Power rebounded "db_proc" "db_cnt_proc" to "db_1/2_proc" "db_1/2_cnt_proc" (this was ignored on older versions) 
--    - rename CLK constrains osc_clk to osc_clk_0 on constrain file.
--------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.STD_LOGIC_arith.all;
use ieee.STD_LOGIC_unsigned.all;
use ieee.STD_LOGIC_misc.all;
library machxo2;
use machxo2.all;
--------------------------------------------------------------------------------
ENTITY top IS
   generic (
    MICOSD_QSPIMODE : integer := 0; -- 0 default (switch to SD Boot Mode, when SD insert), 1:Stay QSPI Boot Mode, whem MicroSD is insert
    DISABLE_POWERDOWN : integer := 0 -- 0 default (power on/off possible), 1:Only power ON (short Pin J10-6 with J10-8 on TEBF0808 to start with power supply on)
   );
PORT (

	--C_TMS		 		: in STD_LOGIC;
	--C_TCK		 		: in STD_LOGIC;
	C_TDI		 		: in STD_LOGIC; --loopback to close chain and make CPLD invisible in FMC JTAG mode
	C_TDO		 		: out  STD_LOGIC; --loopback to close chain and make CPLD invisible in FMC JTAG mode
		
  MIO44       : out STD_LOGIC; --SD WP
    
	MIO42				: out STD_LOGIC;
	MIO43				: in  STD_LOGIC;
  MIO40       : in std_logic; --to PWRLED_P / LED_P
		
		
	XMOD_TXD			: out STD_LOGIC;
	XMOD_RXD			: in  STD_LOGIC;
	XMOD_BTN			: in  STD_LOGIC;
	XMOD_LED			: out  STD_LOGIC;

	SCL					: in  STD_LOGIC;
	SDA					: in  STD_LOGIC;
	PWR_BTN				: in  STD_LOGIC;
	RST_BTN				: in  STD_LOGIC;

	PERSTn				: out STD_LOGIC; --PCIe Reset
	MRESETn				: out STD_LOGIC;
	PROG_B				: INOUT STD_LOGIC;

	-- 1.8V
	MODE0				: out STD_LOGIC;
	MODE1				: out STD_LOGIC;
	MODE2				: out STD_LOGIC;
	MODE3				: out STD_LOGIC;
	ERROR				: in  STD_LOGIC;
	ERR_STAT			: in  STD_LOGIC;
	SRSTn				: out STD_LOGIC;
	DONE				: in  STD_LOGIC;
	INIT				: in  STD_LOGIC;
	PLL_RSTn			: out STD_LOGIC;
	PLL_SEL0			: out STD_LOGIC;
	PLL_SEL1			: out STD_LOGIC;
	PLL_LOL				: in  STD_LOGIC;
	MIO30				: in  STD_LOGIC; --negative per reset from SoC new todo use for periphery resets from SoC| old: force reboot after FSBL-PLL configuration 
	MIO31				: in  STD_LOGIC; --negative pcie reset from SoC
	----------------------
	LED_N				: out STD_LOGIC; 
	LED_P				: out STD_LOGIC; 
		
	HDLED_N				: out STD_LOGIC; 
	HDLED_P				: out STD_LOGIC; 

	-- REV 02
	SD_CD_S				: in  STD_LOGIC; --microSD
	SD_CD_B				: in  STD_LOGIC; --SD
	SD_CD_OUT			: out STD_LOGIC;
	SD_SEL				: out STD_LOGIC;  -- Select microSD or normal SD
	SD_A_EN				: out STD_LOGIC;
	SD_B_EN				: out STD_LOGIC;
		
	FMC_FAN_EN   	: out STD_LOGIC;
  FMC_PG_C2M    : out std_logic;

	SC0					: in  STD_LOGIC;  -- forwarded top LED
	SC1					: in  STD_LOGIC;  -- 
	SC2					: in  STD_LOGIC;  -- 
	-- SC3					: out STD_LOGIC;  -- 
	-- SC4					: out STD_LOGIC;  -- 
	SC5					: out STD_LOGIC;  -- RGPIO
	SC6					: in  STD_LOGIC;  -- RGPIO
	SC7					: in  STD_LOGIC;  -- RGPIO
	-- inter SCx
	X0	 	 			  : in STD_LOGIC;   -- MTS dummy                      --1.8V 
	X1	  				: out STD_LOGIC;  -- inter cpld RGPIO CLK                   --1.8V 
	X2	  				: in STD_LOGIC;  -- inter cpld RGPIO RX                --1.8V 
	X3	  				: out STD_LOGIC;  -- inter cpld RGPIO TX                --1.8V 
	X4	  				: in  STD_LOGIC;  -- MMC SD WP from master              --1.8V 
	X5	  				: out  STD_LOGIC ;  -- STM dummy               --1.8V 
	X6	  				: in  STD_LOGIC;  --  unused     --1.8V 
	X7	  				: in  STD_LOGIC;  --  unused     --1.8V 
	X8	  				: out  STD_LOGIC;  --    --3V3SB
	F1PWM		  		: out STD_LOGIC;
	F1SENSE	 	 		: in  STD_LOGIC;
	AUD_CLK				: out STD_LOGIC; -- -- 24.76MHz Osz used to check 3.3V enabled/disabled and for audio
	MEMS_CLKIN			: in  STD_LOGIC;  -- 
	-- dummy				: out STD_LOGIC; -- 
	----------------------------------------------------
  ------power
	PWROK				: in  STD_LOGIC;  -- power OK from ATX (internal pullup activated to use also on 12V power supply)
	PSON				: out STD_LOGIC; --negative for feedback to ATX
  
	EN_5V				: out  STD_LOGIC; -- Enable 5V, can be permanently enabled by S4-4, no feedback from 5V directly 
  
  ---SoC
	MOD_EN				: out STD_LOGIC; --Enable Module 3.3V Main power
  --PS
	PG_LPD				: in  STD_LOGIC;
	EN_LPD				: out STD_LOGIC;
  
	PG_FPD				: in  STD_LOGIC;
	EN_FPD				: out STD_LOGIC;
  
	EN_DDR				: out STD_LOGIC;
	PG_DDR				: in  STD_LOGIC;
  
	EN_PSGTR			: out STD_LOGIC;
	PG_PSGT				: in  STD_LOGIC;

	PG_PLL				: in  STD_LOGIC;
	EN_PLL_PWR			: out STD_LOGIC;
  --PL
	EN_PL				: out STD_LOGIC;
	PG_PL				: in  STD_LOGIC;
  
	EN_GT_R				: out STD_LOGIC;
	PG_GT_R				: in  STD_LOGIC;
	EN_GT_L				: out STD_LOGIC;
	PG_GT_L				: in  STD_LOGIC;

  ---carrier periphery
	EN_1V8				: out STD_LOGIC;
  POK_1V8       : in std_logic;
  
	PER_EN				: out STD_LOGIC; -- maybe use CLK_CPLD as feedback(24,576MHz on pin 128 CLK_CPLD) 
	PCI_SFP_EN			: out STD_LOGIC;
  
	FMC_EN				: out STD_LOGIC; -- Enable FMC Power DANGER!!
  POK_FMC       : in std_logic;
	----------------------------------------------------
	SW1					: in  STD_LOGIC;
	SW2					: in  STD_LOGIC;
	LED2				: out STD_LOGIC;
	LED3				: OUT STD_LOGIC
);
END ENTITY top;
--------------------------------------------------------------------------------
ARCHITECTURE translated OF top IS

--------------------------------------------------------------------------------
COMPONENT PORESET
PORT(
	SYSRST		: in  STD_LOGIC;
	CLK 		: in  STD_LOGIC;		  
	RST 		: OUT STD_LOGIC
);
END COMPONENT;

component RGPIO_top is
  Generic (
    C_ADD_RESERVED  : integer range 0 to 1          := 0;--0 disabled 1 enabled
    C_TYP           : integer range 0 to 2          := 0  --0 Master, 1 Slave ,2 Both 
  );
  Port ( 
		-----------------------------------------------------
		--------------------FPGAIO-BUS Ports
		--
		-- Master 
		-- -external
		RGPIO_M_CLK   : out std_logic;   --  RGPIO Clock
		RGPIO_M_RX    : in  std_logic;   -- RGPIO RXD        
		RGPIO_M_TX    : out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_M_OUT   : out std_logic_vector(23 downto 0); --user data
    RGPIO_M_IN    : in std_logic_vector(23 downto 0); --user data
		-- -internal control
	  RGPIO_M_RESERVED_OUT            : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_RESERVED_IN             : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_SLAVE_ACTIVATION_CODE   : out  std_logic_vector(3 downto 0); -- activation code from external slavefor information only
	  RGPIO_M_ENABLE                  : in  std_logic; -- Enable RGPIO
	  RGPIO_M_USRCLK                  : in  std_logic; -- RGPIO CLK
	  RGPIO_M_RESET_N                 : in  std_logic; -- RGPIO Reset
		--  
		-- Slave 
		-- -external
		RGPIO_S_CLK   : in std_logic;  --  RGPIO Clock
		RGPIO_S_RX    : in  std_logic;   -- RGPIO RXD        
		RGPIO_S_TX    : out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_S_OUT   : out std_logic_vector(23 downto 0); --user data
    RGPIO_S_IN    : in std_logic_vector(23 downto 0); --user data
		-- -internal control
	  RGPIO_S_RESERVED_OUT  : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_RESERVED_IN   : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_MASTER_ACTIVATION_CODE   : out  std_logic_vector(3 downto 0);  -- activation code from external master for information only
	  RGPIO_S_ENABLED   : out  std_logic -- Enable RGPIO
  
	);
end component RGPIO_top;


COMPONENT OSCH
-- synthesis translate_off
GENERIC (
	NOM_FREQ	: string := "24.18"
);
-- synthesis translate_on
PORT ( 
	STDBY 		: in  STD_LOGIC;
	OSC 		: out STD_LOGIC;
	SEDSTDBY	: out STD_LOGIC
);
END COMPONENT OSCH;
component por_b_rst is 
port ( 
	nreset		: in  STD_LOGIC;
	clk 		: in  STD_LOGIC;
	ce  		: in  STD_LOGIC;
	por_b 		: out STD_LOGIC
);
end component;

 attribute NOM_FREQ : string;
 attribute NOM_FREQ of OSCinst0 : label is "24.18";

constant DB_MAX : integer := 22;
constant DB_MAX2 : integer := 26;

signal osc_clk		: STD_LOGIC;
signal clk			: STD_LOGIC;
signal cnt			: STD_LOGIC_VECTOR(31 downto 0);
signal cnt_pwr		: STD_LOGIC_VECTOR(15 downto 0);
signal usb_reset_n: STD_LOGIC;

signal tick			: STD_LOGIC;
signal blink		: STD_LOGIC;
signal blink_status		: STD_LOGIC_VECTOR(7 downto 0);
signal slow_blink	: STD_LOGIC;
signal rgpio_clk 	: STD_LOGIC; 		
signal rgpio_txd 	: STD_LOGIC; 		
signal rgpio_rxd 	: STD_LOGIC; 		
signal rgpio_out	: STD_LOGIC_VECTOR(23 downto 0); 		
signal rgpio_in 	: STD_LOGIC_VECTOR(23 downto 0); 
signal rgpio_active : STD_LOGIC; 	

signal iclld_rgpio_clk 		: STD_LOGIC; 		
signal iclld_rgpio_txd 		: STD_LOGIC; 		
signal iclld_rgpio_rxd 		: STD_LOGIC; 		
signal iclld_rgpio_out 		: STD_LOGIC_VECTOR(31 downto 0); 		
signal iclld_rgpio_in 		: STD_LOGIC_VECTOR(31 downto 0); 		
signal iclld_rgpio_enable 	: STD_LOGIC; 
signal iclld_uclk 	: STD_LOGIC; 
signal iclld_rstn 	: STD_LOGIC; 
	
signal bmode_i 		    : STD_LOGIC_VECTOR(3 downto 0); 

	
signal SW 			    : STD_LOGIC_VECTOR(1 downto 0); 		
signal SD_detect_n	: STD_LOGIC; 		
signal sd_boot_sel	: STD_LOGIC; 		
signal POR_DONE		: STD_LOGIC;
signal pson_rst_n_i 		: STD_LOGIC;
signal PSON_i 		: STD_LOGIC;
signal led_cnt		: STD_LOGIC_VECTOR(31 downto 0);
signal perstn_i	  : STD_LOGIC;
signal	pwr_btn_i				:std_logic;
signal	pwr_btn_force_i				:std_logic;
signal	rst_btn_sw_i				:std_logic;
signal	rst_btn_pwr_i				:std_logic;

--------------------------------
signal c_pwr_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal n_pwr_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal button_pwr_db : std_logic_vector(3 downto 0) := (others =>'1');
signal db_pwr_rst : std_logic;

signal c_pwr_force_db_cnt : std_logic_vector(DB_MAX2 downto 0) := (others => '0');
signal n_pwr_force_db_cnt : std_logic_vector(DB_MAX2 downto 0) := (others => '0');
signal button_pwr_force_db : std_logic_vector(3 downto 0) := (others =>'1');
signal db_pwr_force_rst : std_logic;

signal c_rst_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal n_rst_db_cnt : std_logic_vector(DB_MAX downto 0) := (others => '0');
signal button_rst_db : std_logic_vector(3 downto 0) := (others =>'1');
signal db_rst_rst : std_logic;

signal c_rst_pwr_db_cnt : std_logic_vector(DB_MAX2 downto 0) := (others => '0');
signal n_rst_pwr_db_cnt : std_logic_vector(DB_MAX2 downto 0) := (others => '0');
signal button_rst_pwr_db : std_logic_vector(3 downto 0) := (others =>'1');
signal db_rst_pwr_rst : std_logic;

----------------------------------
type pwr_state is (IDLE, ATX_ON, V5_ON, MOD_MAIN_ON, V1P8_ON, VPER_ON,WAIT_RDY, RDY);
signal c_p_main_state : pwr_state;-- := IDLE;
signal n_p_main_state : pwr_state;-- := IDLE;

signal pwr_cnt		          : STD_LOGIC_VECTOR(31 downto 0);
signal pwr_down_timeout_cnt : STD_LOGIC_VECTOR(31 downto 0);
signal pwr_down_timeout_per_cnt : STD_LOGIC_VECTOR(31 downto 0);

type pwr_ps_state is (IDLE, MOD_PS_LP_ON, MOD_PS_FPD_ON, MOD_PS_ON, RDY);
signal c_p_mod_ps_state : pwr_ps_state;-- := IDLE;
signal n_p_mod_ps_state : pwr_ps_state;-- := IDLE;

type pwr_pl_state is (IDLE, MOD_PL_ON, RDY);
signal c_p_mod_pl_state : pwr_pl_state;-- := IDLE;
signal n_p_mod_pl_state : pwr_pl_state;-- := IDLE;

signal main_pwr_state : std_logic_vector(3 downto 0); --note only 3 bits used
signal mod_ps_state   : std_logic_vector(3 downto 0); --note only 3 bits used
signal mod_pl_state   : std_logic_vector(3 downto 0); --note only 2 bits used
-----------------------------
signal perihpery_pg : std_logic;
signal pwr_rdy : std_logic;
signal zynq_reset_rdy : std_logic;

signal pwrok_internal : std_logic;
signal pwrok_force_zero_n : std_logic;

--------------------------------------------------------------------------------
signal c_ext_3p3v : std_logic_vector(5 downto 0);
signal n_ext_3p3v : std_logic_vector(5 downto 0);
signal v3p3_off : std_logic;

-- signal mem_clk_led_cnt		: STD_LOGIC_VECTOR(31 downto 0);
signal intercpld_bpwr_i		: STD_LOGIC;

signal fmc_sanity_check_p		: STD_LOGIC;
signal fmc_sanity_check_p_i		: STD_LOGIC_vector(33-1 downto 0);
--------------------------------------------------------------------------------
BEGIN
--------------------------------------------------------------------------------
--unused
-------------
X5 		<=X6 or X7 or X0 or PROG_B  or F1SENSE  or SCL or SDA  or SC1 or SC2 ;
--prog_b not used:
PROG_B 		<=  'Z';


--------------------------------------------------------------------------------
--debounce buttons
-------------
--debounce power button
button_pwr_db(0) <= PWR_BTN;
db_1_proc: process(clk)
begin
  if(rising_edge(clk))then
		button_pwr_db(2 downto 1)		<= button_pwr_db(1 downto 0);
    if(c_pwr_db_cnt(DB_MAX)='1') then
      button_pwr_db(3) <= button_pwr_db(2);
    end if;
	end if;
end process;
db_pwr_rst <= button_pwr_db(2) xor button_pwr_db(1);

db_1_cnt_proc: process(clk,db_pwr_rst)
begin
  if(db_pwr_rst='1') then
    c_pwr_db_cnt <= (others =>'0');
  else
    if rising_edge(clk) then
      c_pwr_db_cnt 	<= n_pwr_db_cnt;
    end if;
  end if;
end process;
n_pwr_db_cnt <= c_pwr_db_cnt + '1' when c_pwr_db_cnt(DB_MAX)='0' else c_pwr_db_cnt;

-----------------------------
--debounce power button force
button_pwr_force_db(0) <= PWR_BTN;
db_2_proc: process(clk)
begin
  if(rising_edge(clk))then
		button_pwr_force_db(2 downto 1)		<= button_pwr_force_db(1 downto 0);
    if(c_pwr_force_db_cnt(DB_MAX2)='1') then
      button_pwr_force_db(3) <= button_pwr_force_db(2);
    end if;
	end if;
end process;
db_pwr_force_rst <= button_pwr_force_db(2) xor button_pwr_force_db(1);

db_2_cnt_proc: process(clk,db_pwr_force_rst)
begin
  if(db_pwr_force_rst='1') then
    c_pwr_force_db_cnt <= (others =>'0');
  else
    if rising_edge(clk) then
      c_pwr_force_db_cnt 	<= n_pwr_force_db_cnt;
    end if;
  end if;
end process;
n_pwr_force_db_cnt <= c_pwr_force_db_cnt + '1' when c_pwr_force_db_cnt(DB_MAX2)='0' else c_pwr_force_db_cnt;

------------------------------
--debounce sw rst button sw reset
button_rst_db(0) <= RST_BTN;
db_3_proc: process(clk)
begin
  if(rising_edge(clk))then
		button_rst_db(2 downto 1)		<= button_rst_db(1 downto 0);
    if(c_rst_db_cnt(DB_MAX)='1') then
      button_rst_db(3) <= button_rst_db(2);
    end if;
	end if;
end process;
db_rst_rst <= button_rst_db(2) xor button_rst_db(1);

db_3_cnt_proc: process(clk,db_rst_rst)
begin
  if(db_rst_rst='1') then
    c_rst_db_cnt <= (others =>'0');
  else
    if rising_edge(clk) then
      c_rst_db_cnt 	<= n_rst_db_cnt;
    end if;
  end if;
end process;
n_rst_db_cnt <= c_rst_db_cnt + '1' when c_rst_db_cnt(DB_MAX)='0' else c_rst_db_cnt;

------------------------------
--debounce sw rst button pwr reset
button_rst_pwr_db(0) <= RST_BTN;
db_4_proc: process(clk)
begin
  if(rising_edge(clk))then
		button_rst_pwr_db(2 downto 1)		<= button_rst_pwr_db(1 downto 0);
    if(c_rst_pwr_db_cnt(DB_MAX2)='1') then
      button_rst_pwr_db(3) <= button_rst_pwr_db(2);
    end if;
	end if;
end process;
db_rst_pwr_rst <= button_rst_pwr_db(2) xor button_rst_pwr_db(1);

db_5_cnt_proc: process(clk, db_rst_pwr_rst)
begin
  if(db_rst_pwr_rst='1') then
    c_rst_pwr_db_cnt <= (others =>'0');
  else
    if rising_edge(clk) then
      c_rst_pwr_db_cnt 	<= n_rst_pwr_db_cnt;
    end if;
  end if;
end process;
n_rst_pwr_db_cnt <= c_rst_pwr_db_cnt + '1' when c_rst_pwr_db_cnt(DB_MAX2)='0' else c_rst_pwr_db_cnt;
------------------------------
pwr_btn_i <= button_pwr_db(3);
pwr_btn_force_i <= button_pwr_force_db(3);
rst_btn_sw_i   <= button_rst_db(3) and  zynq_reset_rdy;     --hold SoC into Reset as long as System power is not ready
rst_btn_pwr_i <= button_rst_pwr_db(3) and zynq_reset_rdy;  --hold SoC into Reset as long as System power is not ready
--------------------------------------------------------------------------------
-- Power/Reset management
-------------
--dummy counter to check 3.3V is available (osc is sourced by 3.3V)
-- at the moment check only power up
check3p3_av: process(MEMS_CLKIN, c_p_main_state) 
begin
  if(c_p_main_state /= VPER_ON )then
    c_ext_3p3v <= (others => '0');
  else 
    if (rising_edge(MEMS_CLKIN)) then
      c_ext_3p3v <= n_ext_3p3v;
    end if;
	end if;
end process;
n_ext_3p3v <=c_ext_3p3v + "000001" when c_ext_3p3v(5)='0' else c_ext_3p3v;
v3p3_off <= '1' when (c_ext_3p3v(5)='0' and c_p_main_state = VPER_ON) else '0';

perihpery_pg <= not v3p3_off; -- periphery 3.3V OK 

-----------------------------
-- Power ON/OFF selection, different variants possible:
-- -1.  ON/OFF with power button (default) -->
--      PSON_i <-- '0' power disabled with pwr_btn_i
--      PSON_i <-- '1' power enabled with pwr_btn_i
-- -2.  Only Power ON -> (short Pin J10-6 with J10-8 on TEBF0808 to start with power supply on)

Inst_PORESET: PORESET 
PORT MAP( 		
	SYSRST 		=> '1', 		
	RST 		=> POR_DONE, 		
	CLK 		=> Clk 	
);

dpd_false: if DISABLE_POWERDOWN=0 generate
  onoff: process(clk)
  begin
    if (rising_edge(clk)) then
      if(POR_DONE = '0')then
        PSON_i <= '0';
      else 
        if pwr_btn_i = '0' and PSON_i = '0' then
          if tick='1' then
            cnt_pwr <= cnt_pwr + X"0001";
            if cnt_pwr = "0100" then
              PSON_i <= '1';
            end if;
          end if;
        end if;
        --Note: uncomment this part and set power button permanent to zero to start with main power on
        if pwr_btn_i = '0' and PSON_i = '1' then
          if tick='1' then
            cnt_pwr <= cnt_pwr + X"0001";
            if cnt_pwr = "0100" then
              PSON_i <= '0';
            end if;
          end if;
        end if;
        if pwr_btn_force_i='0' then
          --reset power on state after hard power down
          PSON_i <= '0';
        end if;
        -- End power down part --------------------------
        if pwr_btn_i = '1' then
          cnt_pwr <= X"0000";
        end if;
      end if;
    
    end if;
  end process;
  pwrok_force_zero_n <= '0' when pwr_btn_force_i='0' and (c_p_main_state /= IDLE) else '1';
end generate;
dpd_true: if DISABLE_POWERDOWN=1 generate
  onoff: process(clk)
  begin
    if (rising_edge(clk)) then
      if(POR_DONE = '0')then
        PSON_i <= '0';
      else 
        if pwr_btn_i = '0' and PSON_i = '0' then
          if tick='1' then
            cnt_pwr <= cnt_pwr + X"0001";
            if cnt_pwr = "0100" then
              PSON_i <= '1';
            end if;
          end if;
        end if;
        if pwr_btn_i = '1' then
          cnt_pwr <= X"0000";
        end if;
      end if;
    
    end if;
  end process;
  pwrok_force_zero_n <= '1';
end generate;
pwrok_internal <= PWROK and pwrok_force_zero_n;
-----------------------------
-- Main Power state machine
-------------

reg_pwr_main_proc: process(clk,pwrok_internal)
begin
  if(pwrok_internal = '0') then
    c_p_main_state <= IDLE;
  else
    if(rising_edge(clk))then
        c_p_main_state		<= n_p_main_state;
    end if;
	end if;
end process;

stat_pwr_main_proc : process(fmc_sanity_check_p,pwr_down_timeout_per_cnt, pwr_down_timeout_cnt,pwr_cnt,pwrok_internal,c_p_main_state,PSON_i,c_p_mod_ps_state,c_p_mod_pl_state,POK_1V8,POK_FMC,perihpery_pg)
begin
  n_p_main_state <= c_p_main_state;
  main_pwr_state <= x"0";
  PSON <= '0'; -- always '0' excepted idle state and PSON_i is zero 
  
  pwr_rdy <= '0';
  zynq_reset_rdy <= '0';
  --add all
	EN_5V				<='0'; -- Enable 5V, can be permanently enabled by S4-4, no feedback from 5V directly 
  ---SoC
	MOD_EN			<='0'; --Enable Module 3.3V Main power
  ---carrier periphery
	EN_1V8				<='0';
  -- POK_1V8       : in std_logic;
  
	PER_EN				<='0'; -- mybe use CLK_CPLD as feedback(24,576MHz on pin 128 CLK_CPLD) 
	PCI_SFP_EN		<='0';
  
	FMC_EN				<='0'; -- Enable FMC Power DANGER!!
  -- POK_FMC       : in std_logic;
  -- fmc_sanity_check_p  --  Sanity check for FMC voltage
  ---
  case c_p_main_state is
    when IDLE =>
      main_pwr_state <= x"0";
      if (PSON_i='1') then
        PSON <= '0';
        if (pwrok_internal = '1') then
          -- on 12V only PWROK is always '1' from internal pullup
          n_p_main_state <= ATX_ON;
        end if;
      else
        PSON <= '1';
      end if;
    when ATX_ON =>
      main_pwr_state <= x"1";
      if (PSON_i='1') then
        EN_5V				<='0';
        MOD_EN			<='0';
        EN_1V8				<='0';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0';
        n_p_main_state <= V5_ON;
      else
        EN_5V				<='0';
        MOD_EN			<='0';
        EN_1V8				<='0';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0';
        n_p_main_state <= IDLE;
      end if;
    when V5_ON =>
      main_pwr_state <= x"2";
      if (PSON_i='1') then
        EN_5V				<='1';
        MOD_EN			<='0';
        EN_1V8				<='0';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0';
        
        n_p_main_state <= MOD_MAIN_ON; -- nothing to check
      else
        EN_5V				<='0';
        MOD_EN			<='0';
        EN_1V8				<='0';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0';
      
        n_p_main_state <= ATX_ON; -- nothing to check
      end if;
    when MOD_MAIN_ON =>
      main_pwr_state <= x"3";
      EN_5V				<='1';
      MOD_EN			<='1';
      EN_1V8				<='0';
      PER_EN				<='0';
      PCI_SFP_EN		<='0';
      FMC_EN				<='0';
      if (PSON_i='1') then
        if (c_p_mod_ps_state=RDY and c_p_mod_pl_state=RDY) then
          n_p_main_state <= V1P8_ON;
        end if;
      else
        if ((c_p_mod_ps_state=IDLE and c_p_mod_pl_state=IDLE) or pwr_down_timeout_cnt(23)='1') then
          EN_5V				<='0';
          n_p_main_state <= V5_ON;
        end if;
      end if;
    when V1P8_ON =>
      main_pwr_state <= x"4";
      if (PSON_i='1') then
        EN_5V				<='1';
        MOD_EN			<='1';
        EN_1V8				<='1';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0';
        if (POK_1V8='1') then
          n_p_main_state <= VPER_ON;
        end if;
      else 
        EN_5V				<='1';
        MOD_EN			<='1';
        EN_1V8				<='0';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0';
        if (POK_1V8='0') then
          n_p_main_state <= MOD_MAIN_ON;
        end if;
      end if;
    when VPER_ON =>
      main_pwr_state <= x"5";
      if (PSON_i='1') then
        EN_5V				<='1';
        MOD_EN			<='1';
        EN_1V8				<='1';
        PER_EN				<='1';
        PCI_SFP_EN		<='1';
        FMC_EN				<= fmc_sanity_check_p; -- --Sanity check from other CPLD 
        if (POK_FMC='1' and perihpery_pg= '1') then
          n_p_main_state <= WAIT_RDY;
        end if;
      else 
        EN_5V				<='1';
        MOD_EN			<='1';
        EN_1V8				<='1';
        PER_EN				<='0';
        PCI_SFP_EN		<='0';
        FMC_EN				<='0'; 
        -- if ( POK_FMC='0' and (perihpery_pg= '0' or pwr_down_timeout_per_cnt(12)='1')) then
        if ( POK_FMC='0' and (pwr_down_timeout_per_cnt(12)='1')) then
          n_p_main_state <= V1P8_ON;
        end if;
      end if;
    when WAIT_RDY =>
      main_pwr_state <= x"6";
      --release main power reset
      pwr_rdy <= '1'; --internal main power ready
      EN_5V				  <='1';
      MOD_EN			  <='1';
      EN_1V8				<='1';
      PER_EN				<='1';
      PCI_SFP_EN		<='1';
      FMC_EN				<= fmc_sanity_check_p; -- --Sanity check from other CPLD 
      if (PSON_i='1' and pwr_cnt(9)='1' ) then
        n_p_main_state <= RDY;
      end if;
      if (PSON_i='0' and pwr_cnt(4)='1' ) then
        n_p_main_state <= VPER_ON;
      end if;
    when RDY =>
      main_pwr_state <= x"7";
      --release zynq reset
      pwr_rdy <= '1'; --internal main power ready
      zynq_reset_rdy <= '1'; --internal zynq main reset
      EN_5V				<='1';
      MOD_EN			<='1';
      EN_1V8				<='1';
      PER_EN				<='1';
      PCI_SFP_EN		<='1';
      FMC_EN				<= fmc_sanity_check_p; -- --Sanity check from other CPLD 
      if (PSON_i='0') then
        n_p_main_state <= WAIT_RDY;
      end if;
    when others =>
  end case;
end process;
-----------------------------
-- Module PS Power state machine
reg_pwr_ps_proc: process(clk,pwrok_internal)
begin
  if(pwrok_internal = '0') then
    c_p_mod_ps_state <= IDLE;
  else
    if(rising_edge(clk))then
        c_p_mod_ps_state		<= n_p_mod_ps_state;
    end if;
	end if;
end process;

stat_pwr_ps_proc : process(c_p_mod_ps_state,PSON_i,c_p_main_state,PG_LPD,PG_FPD,PG_DDR,PG_PSGT, PG_PLL,c_p_main_state)
begin
  n_p_mod_ps_state <= c_p_mod_ps_state;
  --add all
	EN_LPD				<='0';
	-- PG_LPD				: in STD_LOGIC;
  
	EN_FPD				<='0';
	-- PG_FPD				: in STD_LOGIC;
  
	EN_DDR				<='0';
	-- PG_DDR				: in  STD_LOGIC;
  
	EN_PSGTR			<='0';
	-- PG_PSGT				: in  STD_LOGIC;

	EN_PLL_PWR				<='0';
	--PG_PLL 			: in STD_LOGIC;
  
  case c_p_mod_ps_state is
    when IDLE =>
      mod_ps_state <= x"0";
      if (PSON_i='1' and c_p_main_state= MOD_MAIN_ON) then
        n_p_mod_ps_state <= MOD_PS_LP_ON;
      end if;
    when MOD_PS_LP_ON =>
      mod_ps_state <= x"1";
      if (PSON_i='1') then
        EN_LPD				<='1';
        EN_FPD				<='0';
        EN_DDR				<='0';
        EN_PSGTR			<='0';
        EN_PLL_PWR		<='0';
        
        if (PG_LPD='1') then
          n_p_mod_ps_state <= MOD_PS_FPD_ON;
        end if;
      else
        EN_LPD				<='0';
        EN_FPD				<='0';
        EN_DDR				<='0';
        EN_PSGTR			<='0';
        EN_PLL_PWR		<='0';
        if (PG_LPD='0') then
          n_p_mod_ps_state <= IDLE;
        end if;
      end if;
    when MOD_PS_FPD_ON =>
      mod_ps_state <= x"2";
      if (PSON_i='1') then
        EN_LPD				<='1';
        EN_FPD				<='1';
        EN_DDR				<='0';
        EN_PSGTR			<='0';
        EN_PLL_PWR		<='0';
        if (PG_FPD='1') then
          n_p_mod_ps_state <= MOD_PS_ON;
        end if;
      else
        EN_LPD				<='1';
        EN_FPD				<='0';
        EN_DDR				<='0';
        EN_PSGTR			<='0';
        EN_PLL_PWR		<='0';
        if (PG_FPD='0') then
          n_p_mod_ps_state <= MOD_PS_LP_ON;
        end if;
      end if;
    when MOD_PS_ON =>
      mod_ps_state <= x"3";
      if (PSON_i='1') then
        EN_LPD				<='1';
        EN_FPD				<='1';
        EN_DDR				<='1';
        EN_PSGTR			<='1';
        EN_PLL_PWR		<='1';
        if (PG_DDR='1' and PG_PSGT='1' and PG_PLL='1') then
          n_p_mod_ps_state <= RDY;
        end if;
      else
        EN_LPD				<='1';
        EN_FPD				<='1';
        EN_DDR				<='0';
        EN_PSGTR			<='0';
        EN_PLL_PWR		<='0';
        if (PG_DDR='0' and PG_PSGT='0' and PG_PLL='0') then
          n_p_mod_ps_state <= MOD_PS_FPD_ON;
        end if;
      end if;
    when RDY =>
      mod_ps_state <= x"4";
      EN_LPD				<='1';
      EN_FPD				<='1';
      EN_DDR				<='1';
      EN_PSGTR			<='1';
      EN_PLL_PWR		<='1';
      if (PSON_i='0' and c_p_main_state= MOD_MAIN_ON) then
        n_p_mod_ps_state <= MOD_PS_ON;
      end if;
    when others =>
  end case;
end process;
-----------------------------
-- Module PL Power state machine
reg_pwr_pl_proc: process(clk,pwrok_internal)
begin
  if(pwrok_internal = '0') then
    c_p_mod_pl_state <= IDLE;
  else
    if(rising_edge(clk))then
        c_p_mod_pl_state		<= n_p_mod_pl_state;
    end if;
	end if;
end process;

stat_pwr_pl_proc : process(c_p_mod_pl_state,PSON_i,c_p_main_state,PG_PL,PG_GT_R,PG_GT_L)
begin
  n_p_mod_pl_state <= c_p_mod_pl_state;
  --add all
	EN_PL					<='0';
	-- PG_PL				: in  STD_LOGIC;
  
	EN_GT_R					<='0';
	-- PG_GT_R				: in  STD_LOGIC;
	EN_GT_L					<='0';
	-- PG_GT_L				: in  STD_LOGIC;

  case c_p_mod_pl_state is
    when IDLE =>
      mod_pl_state <= x"0";
      if (PSON_i='1' and c_p_main_state= MOD_MAIN_ON) then
        n_p_mod_pl_state <= MOD_PL_ON;
      end if;
    when MOD_PL_ON =>
      mod_pl_state <= x"1";
      if (PSON_i='1') then
        EN_PL					<='1';
        EN_GT_R					<='1';
        EN_GT_L					<='1';
        if (PG_PL='1' and PG_GT_R='1' and PG_GT_L='1') then
          n_p_mod_pl_state <= RDY;
        end if;
      else
        EN_PL					<='0';
        EN_GT_R					<='0';
        EN_GT_L					<='0';
        if (PG_PL='0' and PG_GT_R='0' and PG_GT_L='0') then
          n_p_mod_pl_state <= IDLE;
        end if;
      end if;
    when RDY =>
      mod_pl_state <= x"2";
      EN_PL					<='1';
      EN_GT_R					<='1';
      EN_GT_L					<='1';
      if (PSON_i='0' and c_p_main_state= MOD_MAIN_ON) then
        n_p_mod_pl_state <= MOD_PL_ON;
      end if;
    when others =>
  end case;
end process;

--------------------------------------------------------------------------------
-- Resets
-------------
SRSTn 		<= rst_btn_sw_i; --PS reset (PS_SRST_B)

MRESETn		<= rst_btn_pwr_i; --PS reset  (PS_POR_B)

intercpld_bpwr_i <= POK_1V8;-- 1.8V inter clpd bank voltage disabled
X8 			<= intercpld_bpwr_i; --did not use inter CPLD signals as long as 1.8V is not available on slave activation code is ok

pson_rst_n_i <= pwr_rdy; -- reset

PLL_RSTn 	<= pson_rst_n_i; --module pll only power on reset to have stable clk during zynq programming

usb_reset_n  <= pson_rst_n_i and MIO30;

perstn_i	<= pson_rst_n_i and MIO31; --Pci connector --> 
PERSTn    <= perstn_i;

--------------------------------------------------------------------------------
--Si5345 of module
-------------
PLL_SEL0 	<= '0';
PLL_SEL1 	<= '0';
--------------------------------------------------------------------------------
-- internal CLKs and counter
-------------
--internal CLK
OSCInst0: OSCH
-- synthesis translate_off
GENERIC MAP ( 
	NOM_FREQ => "24.18" 
)
-- synthesis translate_on
PORT MAP (
	STDBY		=> '0', 
	OSC			=> osc_clk, 
	SEDSTDBY	=> open
);
clk 	<= osc_clk;
-----------------------------
-- used to release zynq reset with small delay after power good
pwr_counter:  process(clk)
begin
	if rising_edge(clk) then
    if c_p_main_state /= WAIT_RDY then
      pwr_cnt <= (others => '0');
    else
      pwr_cnt 	<= pwr_cnt + X"00000001";
    end if;
	end if;
end process;
-----------------------------
-- used to power down module, if regulator is not assembled and PG is always on with pullup
pwrtimeout_counter: process(clk)
begin
	if rising_edge(clk) then
    if c_p_main_state /= MOD_MAIN_ON then
      pwr_down_timeout_cnt <= (others => '0');
    else
      pwr_down_timeout_cnt 	<= pwr_down_timeout_cnt + X"00000001";
    end if;
	end if;
end process;
-----------------------------
-- used to power down periphery
waitper_counter: process(clk)
begin
	if rising_edge(clk) then
    if c_p_main_state /= VPER_ON then
      pwr_down_timeout_per_cnt <= (others => '0');
    else
      pwr_down_timeout_per_cnt 	<= pwr_down_timeout_per_cnt + X"00000001";
    end if;
	end if;
end process;
-----------------------------
--used to generate power on or off condition
tick_conter: process(clk)
begin
	if (rising_edge(clk)) then
		cnt <= cnt + X"00000001";
		if cnt(17 downto 0) = "100000000000000000" then 
			tick <= '1';
		else
			tick <= '0';
		end if;			
	end if;
end process;
-----------------------------
-- only for all led blinking sequences
led_conter: process(clk)
begin
	if rising_edge(clk) then
		led_cnt 	<= led_cnt + X"00000001";
	end if;
end process;

blink 		  <= cnt(22); --fast --> same as blink_status(6)
slow_blink  <= cnt(25); --slow

blink_status(0) <= '0';                                                             -- off
blink_status(1) <= led_cnt(25) and led_cnt(24) and led_cnt(23) and led_cnt(22);		  -- *ooooooo
blink_status(2) <= led_cnt(25) and led_cnt(24) and led_cnt(22);						          -- **oooooo
blink_status(3) <= led_cnt(25) and (led_cnt(24) or led_cnt(23)) and led_cnt(22);	  -- ***ooooo
blink_status(4) <= led_cnt(25) and led_cnt(22);										                  -- ****oooo
blink_status(5) <= (led_cnt(25) or (led_cnt(24) and led_cnt(23))) and led_cnt(22);	-- *****ooo
blink_status(6) <= led_cnt(22);                                                     -- ********
blink_status(7) <= '1';                                                             -- on

--------------------------------------------------------------------------------
--JTAG loopback CPLD is invisible and chain is cosed if CPLD JTAG is disabled 
-------------
C_TDO	<= C_TDI;
--------------------------------------------------------------------------------
--Boot Mode
-------------
-- 0000 JTAG
-- 0010 SPI32
-- 0101 SD1
-- 0110 eMMC
-- 1000 PJTAG0
SW(0) 		<= SW1;
SW(1) 		<= SW2;

bmode_i	   <= 
	"0101" when  (SW = "00" and sd_boot_sel='0') else 
	"0010" when  (SW = "00" and sd_boot_sel='1')  else  -- SPI when no SD Card
	"0110" when  (SW = "01")  else  -- eMMC
	"1000" when  (SW = "10")  else -- PJTAG0
	"0000"; -- JTAG MODE

MODE0 		<= bmode_i(0);
MODE1 		<= bmode_i(1);
MODE2 		<= bmode_i(2);
MODE3 		<= bmode_i(3);

--------------------------------------------------------------------------------
-- FMC Connector
-------------
FMC_PG_C2M <= pwr_rdy and POK_FMC; --FMC Power good

FMC_FAN_EN	<= pson_rst_n_i; --FMC FAN

--------------------------------------------------------------------------------
-- Main fan from enclosure. Enabled with 3.3V_PER
F1PWM <= '1'; --todo on next firmware update: add pwm control
--------------------------------------------------------------------------------
--LEDs
-------------
--LED2 is D6 Green
LED2      <=  blink_status(6) when main_pwr_state=x"0"                                                                                  else    -- ******** -- power off
              blink_status(5) when PG_LPD='0'                                                                                           else    -- *****ooo
              blink_status(4) when PG_FPD='0'                                                                                           else    -- ****oooo
              blink_status(3) when PG_PL='0'                                                                                            else    -- ***ooooo
              blink_status(2) when PG_DDR='0'  or PG_PSGT='0' or PG_PLL='0' or PG_GT_L='0' or PG_GT_R='0'                               else    -- **oooooo
              blink_status(1) when POK_1V8='0' or POK_FMC='0' or perihpery_pg= '0' or (main_pwr_state=x"7" and fmc_sanity_check_p='0')  else    -- *ooooooo
              blink_status(0) when main_pwr_state=x"7"                                                                                  else    -- off      --power ok  
              blink_status(7) ;                                                                                                                  -- on       -- ERROR some power up failed 
                  
--LED3 is  D7 Red         
LED3 		  <=  blink_status(7) when rst_btn_pwr_i= '0' else    -- on       --reset button pressed long time(also on power off state) and zynq reset
              blink_status(0) when rst_btn_sw_i = '0'  else   -- off      --reset button pressed short time and zynq reset
              blink_status(1) when bmode_i = "0101"   else    -- *ooooooo -- SD Boot        
              blink_status(2) when bmode_i = "0010"   else    -- **oooooo -- SPI Boot                           
              blink_status(3) when bmode_i = "0110"   else    -- ***ooooo -- eMMC             
              blink_status(4) when bmode_i = "1000"   else    -- ****oooo -- PJTAG0             
              blink_status(5) when bmode_i = "0000"   else    -- *****ooo -- JTAG            
              blink_status(6) ;                               -- ******** -- ERROR should never be here                                                                                          
                 

--power led on enclosure (light bulb) 
LED_N 		<=  '0';
LED_P 		<=  blink_status(6) when pwr_btn_force_i='0'                else  -- ********      --power button pressed long time forced power down
              slow_blink      when PSON_i='0' and main_pwr_state=x"0" else  -- slow blinking --power off
              blink_status(5) when pwr_btn_i='0'                      else  -- *****ooo      --power button pressed short time power on/off 
              blink_status(4) when button_rst_pwr_db(3)= '0'          else  -- ****oooo      --reset button pressed long time
              blink_status(3) when button_rst_db(3) = '0'             else  -- ***ooooo      --reset button pressed short time 
              blink_status(2) when PSON_i='0'                         else  -- **oooooo      --power down sequencing is running ()
              blink_status(1) when pson_rst_n_i = '0'                 else  -- *ooooooo      --system hold into reset
              MIO40 ;              --depends on user design

--HD led on enclosure (drive) 
HDLED_N		<=  '0';
HDLED_P		<=  blink_status(6) when INIT='0'      else  -- ********
              blink_status(5) when ERROR = '1'   else  -- *****ooo
              blink_status(4) when ERR_STAT='1'  else  -- ****oooo
              blink_status(3) when DONE='0'      else  -- ***ooooo
           -- blink_status(2) when DONE='0'      else  -- **oooooo  --unused
           -- blink_status(1) when DONE='0'      else  -- *ooooooo  --unused
              SC0;                                 --depends on user design

--XMOD with green dot red LED --> XMOD Button inverts signal
 
XMOD_LED  <=  blink_status(6) when XMOD_BTN='0' and pwr_btn_force_i='0'                         else    -- ******** --power button pressed long time forced power down
              blink_status(5) when XMOD_BTN='0' and PSON_i='0'  and main_pwr_state=x"0"         else    -- *****ooo --power down 
              blink_status(4) when XMOD_BTN='0' and pwr_btn_i='0'                               else    -- ****oooo --power button pressed
              blink_status(3) when XMOD_BTN='0' and button_rst_db(3)= '0'                       else    -- ***ooooo -- reset button pressed
              blink_status(2) when XMOD_BTN='0' and PSON_i='0'                                  else    -- **oooooo --power down sequencing active
              blink_status(1) when XMOD_BTN='0' and pson_rst_n_i = '0'                          else    -- *ooooooo --power on reset
              blink_status(7) when XMOD_BTN='0' and INIT='0'                                    else    -- off (inverted from pressed XMOD Button)  -- PS init low
              not blink_status(6) when XMOD_BTN='1' and pwr_rdy='1' and fmc_sanity_check_p='0'  else    -- ******** --problem with other CPLD, FMC is disabled
              not blink_status(5) when XMOD_BTN='1' and main_pwr_state=x"6"                     else    -- *****ooo  --power state 6 (Wait ready)
              not blink_status(4) when XMOD_BTN='1' and main_pwr_state=x"5"                     else    -- ****oooo  --power state 5 (Periphery Power on/off)
              -- not blink_status(3) when XMOD_BTN='1' and                                      else    -- ***ooooo  --unused
              not blink_status(2) when XMOD_BTN='1' and  pwr_rdy='0'                            else    -- **oooooo  --power not ready(power reset)
              not blink_status(1) when XMOD_BTN='1' and  zynq_reset_rdy='0'                     else    -- *ooooooo  --zynq reset
              blink_status(0) when XMOD_BTN='1' and INIT='0'                                    else    -- off -- PS init low
              blink_status(0) when XMOD_BTN='0'                                                 else    -- on  --all fine (if XMOD Button is press) 
              blink_status(7);                                                                       -- on  --all fine (if XMOD Button is not press)  
                     
-- XMOD_LED  <=  not blink_status(6) when XMOD_BTN='0' and pwr_rdy='1' and fmc_sanity_check_p='0' else    -- ******** --problem with other CPLD, FMC is disabled
              -- not blink_status(1) when XMOD_BTN='0' and mod_ps_state=x"1"                      else    -- *ooooooo
              -- not blink_status(2) when XMOD_BTN='0' and mod_ps_state=x"2"                      else    -- **oooooo
              -- not blink_status(3) when XMOD_BTN='0' and mod_ps_state=x"3"                      else    -- ***ooooo
              -- not blink_status(4) when XMOD_BTN='0' and mod_pl_state=x"1"                      else    -- ****oooo
              -- -- blink_status(5) when XMOD_BTN='0' and mod_pl_state=x"1"                   else    -- *****ooo --unused
              -- blink_status(0) when XMOD_BTN='0' and INIT='0'                               else    -- on (inverted from pressed XMOD Button) 
              -- blink_status(1) when XMOD_BTN='1' and main_pwr_state=x"1"  else    -- *ooooooo
              -- blink_status(2) when XMOD_BTN='1' and main_pwr_state=x"2"  else    -- **oooooo
              -- blink_status(3) when XMOD_BTN='1' and main_pwr_state=x"3"  else    -- ***ooooo
              -- blink_status(4) when XMOD_BTN='1' and main_pwr_state=x"4"  else    -- ****oooo
              -- blink_status(5) when XMOD_BTN='1' and main_pwr_state=x"5"  else    -- *****ooo
              -- blink_status(6) when XMOD_BTN='1' and main_pwr_state=x"6"  else    -- ********
              -- blink_status(7) when XMOD_BTN='1' and INIT='0'             else    -- on
              -- blink_status(0) when XMOD_BTN='0'                          else    -- off all fine (if XMOD Button is press) 
              -- blink_status(7);                                                   -- off all fine (if XMOD Button is not press)    

--------------------------------------------------------------------------------
--Audio CLK
-------------
AUD_CLK 	<= MEMS_CLKIN;
--------------------------------------------------------------------------------
-- SD, different variants possible:
-- -1.  --change boot Mode to SD, if SD is insered(default), 
-- -2.  --change only on normal SD to SD boot, --> microSD can be used for linux and as file system only 
-------------
SD_detect_n <= SD_CD_S and SD_CD_B; --Any SD inserted?
msdboot_enable: if MICOSD_QSPIMODE = 0 generate
  sd_boot_sel <= SD_CD_S and SD_CD_B; --change boot mode in both cases to SD
end generate;
msdboot_disable: if MICOSD_QSPIMODE = 1 generate
  sd_boot_sel <=  SD_CD_B; --change only on normal SD to SD boot, --> microSD can be used for linux and as filesystem only 
end generate;
SD_SEL		<= '0' when SD_CD_S = '0' else '1'; -- MicroSD has higher priority!
SD_CD_OUT	<= SD_detect_n; -- DETECTED
SD_A_EN		<= pson_rst_n_i;
SD_B_EN		<= pson_rst_n_i;

MIO44 <= '0' when SD_CD_S = '0' else X4;
--------------------------------------------------------------------------------
-- UART	
-------------
XMOD_TXD	<= '1' when pwr_rdy='0' else MIO43 ;
MIO42 		<= '1' when pwr_rdy='0' else XMOD_RXD; --
--------------------------------------------------------------------------------
-- SoC/CPLD RGPIO 
-------------
-- rgpio_out(23 downto 8) currently not used
rgpio_in(0) <= SW1; -- S5-1
rgpio_in(1) <= SW2; -- S5-2
rgpio_in(2) <= XMOD_BTN; -- extra button G
rgpio_in(3) <= '1';--unused
rgpio_in(4) <= SD_CD_S; --
rgpio_in(5) <= SD_CD_B; --
rgpio_in(6) <= ERROR; --    PS ERROR
rgpio_in(7) <= ERR_STAT; -- PS ERROR_STAT
rgpio_in(11 downto 8) <= bmode_i; --current boot mode
rgpio_in(12)  <= PG_LPD; --
rgpio_in(13)  <= PG_DDR; --
rgpio_in(14)	<= PG_FPD; 
rgpio_in(15)	<= PG_PSGT; 
rgpio_in(16)	<= PG_GT_L; 
rgpio_in(17)	<= PG_GT_R; 
rgpio_in(18)	<= POK_FMC; 
rgpio_in(19)	<= fmc_sanity_check_p; 
rgpio_in(20)	<= pwrok_internal; 
rgpio_in(21)	<= PG_PL; 
rgpio_in(22)	<= PG_PLL; 
rgpio_in(23)	<= PLL_LOL; 

rgpio_clk 	<= SC6; 
SC5 		<= rgpio_txd;
rgpio_rxd 	<= SC7;

---------------------------
SoC_MyRemoteGPIO_slave : RGPIO_top 
  Generic map(
    C_ADD_RESERVED  => 1,--0 disabled 1 enabled
    C_TYP           => 1  --0 Master, 1 Slave ,2 Both 
  )
  Port map( 
		-----------------------------------------------------
		--------------------FPGAIO-BUS Ports
		--
		-- Master 
		-- -external
		RGPIO_M_CLK   => open,--: out std_logic;   --  RGPIO Clock
		RGPIO_M_RX    =>'1',--: in  std_logic;   -- RGPIO RXD        
		RGPIO_M_TX    => open,--: out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_M_OUT                   => open,             --: out std_logic_vector(23 downto 0); --user data
    RGPIO_M_IN                    => (others => '1'),  --: in std_logic_vector(23 downto 0); --user data
		-- -internal controll         => ,
	  RGPIO_M_RESERVED_OUT          => open,  --: out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_RESERVED_IN           => (others => '1'),  --: in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_SLAVE_ACTIVATION_CODE => open,  --: out  std_logic_vector(3 downto 0); -- activation code from external slavefor information only
	  RGPIO_M_ENABLE                => '1',  --: in  std_logic; -- Enable RGPIO
	  RGPIO_M_USRCLK                => '1',  --: in  std_logic; -- RGPIO CLK
	  RGPIO_M_RESET_N               => '1',  --: in  std_logic; -- RGPIO Reset
		--  
		-- Slave 
		-- -external
		RGPIO_S_CLK   => rgpio_clk,--: in std_logic;  --  RGPIO Clock
		RGPIO_S_RX    => rgpio_rxd,--: in  std_logic;   -- RGPIO RXD        
		RGPIO_S_TX    => rgpio_txd,--: out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_S_OUT                     => rgpio_out(23 downto 0), -- : out std_logic_vector(23 downto 0); --user data
    RGPIO_S_IN                      => rgpio_in(23 downto 0),  -- : in std_logic_vector(23 downto 0); --user data
		-- -internal control        
	  RGPIO_S_RESERVED_OUT            =>  open, -- : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_RESERVED_IN             =>  "1111", --  : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_MASTER_ACTIVATION_CODE  => open, --- : out  std_logic_vector(3 downto 0);  -- activation code from external master for information only
	  RGPIO_S_ENABLED                 => rgpio_active -- : out  std_logic -- Enable RGPIO
  
	);


--------------------------------------------------------------------------------
--Inter CPLD RGPIO runs only if 1.8V is enabled (bank io power)
-------------
-- iclld_rgpio_out(27 downto 3) -- unused
fmc_sanity_check_p_i(0)     <='0'    when intercpld_bpwr_i='0' else iclld_rgpio_out(1) and (not iclld_rgpio_out(0)); --FMC VADJ sanity check

delay_reg: process(clk)
begin
  if(rising_edge(clk)) then
    fmc_sanity_check_p_i(fmc_sanity_check_p_i'length-1 downto 1)<= fmc_sanity_check_p_i(fmc_sanity_check_p_i'length-2 downto 0);
  end if;
end process;  
fmc_sanity_check_p <= and_reduce(fmc_sanity_check_p_i);

iclld_rgpio_in(27 downto 24) <= x"F";
iclld_rgpio_in(23)            <= not rst_btn_sw_i;   --neg ps soft reset     -->invert all to pos 
iclld_rgpio_in(22)            <= not rst_btn_pwr_i;  --neg ps power on reset -->invert all to pos
iclld_rgpio_in(21)            <= ERROR;              --pos Err flag          -->invert all to pos
iclld_rgpio_in(20)            <= ERR_STAT;           --pos Err stat flag     -->invert all to pos
iclld_rgpio_in(19)            <= not INIT;           --neg Init flag         -->invert all to pos
iclld_rgpio_in(18)            <= not perstn_i;       --neg pcie reset        -->invert all to pos
iclld_rgpio_in(17)            <= not usb_reset_n;    --neg usb reset         -->invert all to pos
iclld_rgpio_in(16)            <= not pson_rst_n_i;   --neg power on reset    -->invert all to pos
iclld_rgpio_in(15)            <= pson_rst_n_i;       --neg power on reset    -->c send both for main reset
iclld_rgpio_in(14 downto 12)  <= main_pwr_state(2 downto 0); -- main power state machine | only correct if 1.8 is enabled
iclld_rgpio_in(11 downto 8)   <= bmode_i;                    -- Boot Mode                | only correct if 1.8 is enabled
iclld_rgpio_in(7 downto 0)    <= rgpio_out(7 downto 0) when rgpio_active='1' else x"00"; --test purpose from fpga to other CPLD and back to FPGA

X1 <= iclld_rgpio_clk ; 
iclld_rgpio_rxd  <= X2;
X3 <= iclld_rgpio_txd;  
iclld_uclk <= clk;
iclld_rstn <= '1';
iclld_rgpio_enable <= intercpld_bpwr_i;


-------------------------
InterCPLD_MyRemoteGPIO_master : RGPIO_top 
Generic map(
  C_ADD_RESERVED  => 1,--0 disabled 1 enabled
  C_TYP           => 0  --0 Master, 1 Slave ,2 Both 
)
Port map( 
  -----------------------------------------------------
  --------------------FPGAIO-BUS Ports
  --
  -- Master 
  -- -external
  RGPIO_M_CLK   => iclld_rgpio_clk, --: out std_logic;   --  RGPIO Clock
  RGPIO_M_RX    => iclld_rgpio_rxd, --: in  std_logic;   -- RGPIO RXD        
  RGPIO_M_TX    => iclld_rgpio_txd, --: out std_logic;   -- RGPIO TXD 
  
  -- -internal data
  RGPIO_M_OUT   => iclld_rgpio_out(23 downto 0), --: out std_logic_vector(23 downto 0); --user data
  RGPIO_M_IN    => iclld_rgpio_in(23 downto 0), -- : in std_logic_vector(23 downto 0); --user data
  -- -internal control
  RGPIO_M_RESERVED_OUT            => iclld_rgpio_out(27 downto 24), --: out  std_logic_vector(3 downto 0); -- reserved for future usage
  RGPIO_M_RESERVED_IN             => iclld_rgpio_in(27 downto 24), -- : in  std_logic_vector(3 downto 0); -- reserved for future usage
  RGPIO_M_SLAVE_ACTIVATION_CODE   => iclld_rgpio_out(31 downto 28), --: out  std_logic_vector(3 downto 0); -- activation code from external slavefor information only
  RGPIO_M_ENABLE                  => iclld_rgpio_enable, --: in  std_logic; -- Enable RGPIO
  RGPIO_M_USRCLK                  => iclld_uclk,--: in  std_logic; -- RGPIO CLK
  RGPIO_M_RESET_N                 => iclld_rstn,--: in  std_logic; -- RGPIO Reset
  --  
  -- Slave 
  -- -external
  RGPIO_S_CLK   => '1',--: in std_logic;  --  RGPIO Clock
  RGPIO_S_RX    => '1',--: in  std_logic;   -- RGPIO RXD        
  RGPIO_S_TX    =>  open,--: out std_logic;   -- RGPIO TXD 
  
  -- -internal data
  RGPIO_S_OUT   => open, --: out std_logic_vector(23 downto 0); --user data
  RGPIO_S_IN    => (others => '1'), --: in std_logic_vector(23 downto 0); --user data
  -- -internal control
  RGPIO_S_RESERVED_OUT              => open , --: out  std_logic_vector(3 downto 0); -- reserved for future usage
  RGPIO_S_RESERVED_IN               => (others => '1') , --: in  std_logic_vector(3 downto 0); -- reserved for future usage
  RGPIO_S_MASTER_ACTIVATION_CODE    => open , --: out  std_logic_vector(3 downto 0);  -- activation code from external master for information only
  RGPIO_S_ENABLED                   => open  --: out  std_logic -- Enable RGPIO

);

--------------------------------------------------------------------------------
END ARCHITECTURE translated;